#ifndef JABATOBLOCK_H
#define JABATOBLOCK_H

#include <qstringlist.h>
#include <qpointarray.h>
#include <qstring.h>

#include "jabatoSound.h"

#ifdef DEBUG
# undef DEBUG
#endif
//#define DEBUG(x)   (x)
#define DEBUG(x)

#define PRINTDIRS(x) (x)
//#define PRINTDIRS(x)

class JabatoStrip {
public:
  JabatoStrip();

  int blockId;
  int dayOfTheWeek;
  int beginAtHour;
  int beginAtMinute;
  int endAtHour;
  int endAtMinute;

  bool recordingStrip;

  bool withinStrip(int dow, int hh, int mm) const;
};


class JabatoDirDigger {
public:
  JabatoDirDigger();
  ~JabatoDirDigger();

  void addDirectory(const char *dirpath, char *filter);

  void init(bool shuffle);

  bool isShuffled();

  uint count() const;
  char *getSound();

  char *getSoundAt(int i);

  void clear();

private:
  QStrList    soundList;
  QArray<int> accessArray;
  uint	      nextSound;
  bool        bShuffle;

  bool        alreadyInArray(int index, int upTo);

  void 				checkIndexes();
};

class JabatoBlock {
public:
  JabatoBlock(int id, char *name);
  ~JabatoBlock();

  void  setId(int id);
  void  setName(char *);
  void  setHRef(char *);
  void  setJabatos(char *);
  void  setImage(char *);
  void  setBackImage(char *);
  void  setEmail(char *);
  void  setDescription(char *);

  int   getId() const;
  const char *getName() const;
  const char *getHRef() const;
  const char *getJabatos() const;
  const char *getImage() const;
  const char *getBackImage() const;
  const char *getEmail() const;
  const char *getDescription() const;

  bool  isLoaded() const;

  void setProgramFile(char *);
  const char *getProgramFile() const;
  void setProgramToLanDiskCmd(char *);
  void setProgramToWanDiskCmd(char *);

  void addSongsDirectory(char *, char *);
  void addSlotsDirectory(char *, char *);
  void addTextsDirectory(char *, char *);

  void setSongsPerSlot(int);

  const int  getSongsPerSlot();

  void setShuffleType(int);  // 0 - NO SHUFFLE
                             // 1 - ONLY SONGS
                             // 2 - ONLY SLOTS
                             // 3 - SONGS&SLOTS
			     // 4 - ONLY TEXTS
			     // 5 - TEXT&SONGS
			     // 6 - TEXT&SLOTS
			     // 7 - TEXT&SLOTS&MUSIC
  bool songsShuffled();
  bool slotsShuffled();
  bool textsShuffled();

  JabatoDirDigger *getSongsBlock();
  JabatoDirDigger *getSlotsBlock();
  JabatoDirDigger *getTextsBlock();
  void setSongsBlock(JabatoDirDigger *);
  void setSlotsBlock(JabatoDirDigger *);
  void setTextsBlock(JabatoDirDigger *);

  uint songsCount() const;
  uint slotsCount() const;
  uint textsCount() const;

  void updateSound(JabatoSound *);

  void init();
  void clear();

private:
  int  m_id;
  QString m_name;
  QString m_href;
  QString m_jabatos;
  QString m_image;
  QString m_backimage;
  QString m_email;
  QString m_description;

  QString program;
  QString programToLanDiskCmd;
  QString programToWanDiskCmd;

  JabatoDirDigger *songsBlock, *slotsBlock, *textsBlock;

  int songsPerSlot, songsPlayed;

  int shuffleType;

  bool programPlayed;
  bool textTold;
};

#endif
