#include <stdio.h>
#include <stdlib.h>

#include <util/PlatformUtils.hpp>
#include <sax2/SAX2XMLReader.hpp>
#include <sax2/XMLReaderFactory.hpp>

#include <qmessagebox.h>

//#include <klocale.h>
//#define _(TEXT) globalKlocale->translate(TEXT)

#include "jabatoSchedule.h"

#include <qdir.h>
//#include <qfileinfo.h>

/* ---------------------------------------------------------------------- */
JabatoSchedule::JabatoSchedule() {
  stationName = 0;
  autoRecording = false;
  samplingFrequency = 0;
  jSFile = 0;
  jBFile = 0;
  blocks.setAutoDelete(TRUE);
  externalParser = false;
}


JabatoSchedule::~JabatoSchedule() {
  if(jBFile||jSFile) {
    delete blockH;
    delete scheduleH;
    if (jBFile) delete jBFile;
    if (jSFile) delete jSFile;
    if (stationName) free(stationName);
    if(!externalParser) {
      try
      {
        XMLPlatformUtils::Terminate();
      }
      catch (const XMLException& toCatch)
      {
        cerr << "XMLException during XML parser termination. \n";
      }
      catch(...) {
        cerr << "ERROR during XML parser termination. \n";
      }
    }
  }
}


void
JabatoSchedule::findJabatoFiles() {

  QFileInfo  *filename;
  QDir  *d;

  //d = new QDir("/etc/kontinuidadJabata","*.jSF",static_cast<QDir::SortSpec>(QDir::Name | QDir::Reversed));
  d = new QDir(CFGDIR,"*.jSF",
	       static_cast<QDir::SortSpec>(QDir::Name | QDir::Reversed));
  if (d->exists()&&d->isReadable())
  {
	  
    QFileInfoListIterator it(*(d->entryInfoList()));
    if ((filename=it.current()))
      jSFile = new QString(d->absFilePath(filename->fileName()).data());
    else {
      cerr << "JabatoSchedule: CANNOT FIND " << CFGDIR << "/*.jSF!!\n";
      return;
    }
  }
  delete d;

  // d = new QDir("/etc/kontinuidadJabata","*.jBF",static_cast<QDir::SortSpec>(QDir::Name | QDir::Reversed));
  d = new QDir(CFGDIR,"*.jBF",static_cast<QDir::SortSpec>(QDir::Name | QDir::Reversed));
  if (d->exists()&&d->isReadable())
  {
    QFileInfoListIterator it(*(d->entryInfoList()));
    if ((filename=it.current()))
      jBFile = new QString(d->absFilePath(filename->fileName()).data());
    else {
      cerr << "JabatoSchedule: CANNOT FIND " << CFGDIR << "*.jBF!!\n";
      return;
    }
  }
  delete d;
}


bool
JabatoSchedule::initSchedule(SAX2XMLReader *p, bool loadSnds) {
  if (jSFile==NULL) {

    findJabatoFiles();

    if(jSFile && jBFile) {
      if(initParser(p)) {
        loadSounds = loadSnds;
        blockH = new jabatoBlockHandler();
	blocksH = new jabatoBlocksHandler();
	scheduleH = new jabatoScheduleHandler();
        try {
	  parser->setContentHandler(scheduleH);
	  parser->setErrorHandler(scheduleH);
	  scheduleH->loadScheduledTasks();
	  parser->parse(jSFile->data());
          scheduleH->findStationName();
 	  parser->parse(jSFile->data());
	  parser->setContentHandler(blocksH);
	  parser->setErrorHandler(blocksH);
	  blocks.clear();
          blocksH->findBlocks(&blocks);
	  parser->parse(jBFile->data());
  	  if(blocks.count()>0) {
  	    parser->setContentHandler(blockH);
  	    parser->setErrorHandler(blockH);
	    blockH->setAddDirectories(loadSounds);
            blockH->find(0, blocks.at(0));
  	    parser->parse(jBFile->data());
	  }
	  
	  if(scheduleH->found() && blockH->found())
	  {
	    stationName = scheduleH->getStationName();
	    autoRecording = scheduleH->autoRecMode();
	    samplingFrequency = scheduleH->samplingFreq();
	    cerr << "\nJabatoSchedule::initSchedule: "
	         << "Station name: " << stationName << "\n";
	    return true;
	  }
	  else {
	    cerr << "\nJabatoSchedule::initSchedule: "
	         << "Station name or default block missed in "
	         << jSFile->data() << "' or '" << jBFile->data() << "'.\n";
   	    return false;
 	  }
	}
	catch (const XMLException& e) {
          cerr << "\nJabatoSchedule::initSchedule: "
	       << "Error during parsing. Exception message is:  \n"
      	       << StrX(e.getMessage()) << "\n" ;
      	  return false;
	}
	catch (...)
	{
	  cerr << "\nJabatoSchedule::initSchedule: "
	       << "Unexpected exception during parsing '" << jSFile->data()
	       << "' or '" << jBFile->data() << "'.\n";
	  return false;
	}
      }
      else
        return false;
    }
    else {
      cerr << "JabatoSchedule::initSchedule: *.jSF or *.jBF NOT FOUND!!\n";
      return false;
    }
  }
  else
    return true;
}


bool
JabatoSchedule::initParser(SAX2XMLReader *p) {

  if(p==NULL) {
    try
    {
        XMLPlatformUtils::Initialize();
        parser = XMLReaderFactory::createXMLReader();
        return true;
    }
    catch (const XMLException& toCatch)
    {
        cerr << "Error during initialization! Message:\n"
            << StrX(toCatch.getMessage()) << endl;
        return false;
    }
  }
  else {
    parser = p;
    externalParser=true;
    return true;
  }
}


JabatoBlock *
JabatoSchedule::getDefaultBlock() {
  DEBUG(fprintf(stderr, "JabatoSchedule::getDefaulBlock()\n"));
  return blocks.at(0);
}


JabatoBlock *
JabatoSchedule::getBlockAt(int index) {
  DEBUG(fprintf(stderr, "JabatoSchedule::getBlockAt(%d):", index));

  if((index<(int)blocks.count()) && (index>-1)) {
    
    // RECARGA DE BLOQUES EN CACHE
    if (index>0 && blocks.at(index)->isLoaded()) {
      if(blocks.at(0)->getSongsBlock()!=blocks.at(index)->getSongsBlock()) { 
        blocks.at(index)->getSongsBlock()->clear();
        if(blocks.at(0)->getSlotsBlock()!=blocks.at(index)->getSlotsBlock())
          blocks.at(index)->getSlotsBlock()->clear();
        blocks.at(index)->getTextsBlock()->clear();
      }
    }
    // HASTA AQUÍ

    if(blocks.at(index)->isLoaded()) {
      DEBUG(fprintf(stderr, " %s\n",blocks.at(index)->getName()));
      blocks.at(index)->init();
      return blocks.at(index);
    }
    else {
      DEBUG(fprintf(stderr, " loading%s...\n",loadSounds?" sounds":""));

      blockH->setAddDirectories(loadSounds);
      blockH->find(blocks.at(index)->getId(), blocks.at(index));
      try
      {
  	parser->setContentHandler(blockH);
    	parser->setErrorHandler(blockH);
        parser->parse(jBFile->data());
      }
      catch (const XMLException& e)
      {
        cerr << "Error during parsing: '" << jBFile->data() 
	     << "'\n" << "Exception message is:  \n"
	     << StrX(e.getMessage()) << "\n" ;
      }
      catch (...)
      {
      	cerr << "\nUnexpected exception during parsing: '" 
	     << jBFile->data() << "'\n";
      }

      if (blockH->error()) {
      	cerr << "\nError during parsing: '" << jBFile->data() << "'\n";
        return blocks.at(0);
      }
      else if (!blockH->found()) {
      	cerr << "\nBlock id " << blocks.at(index)->getId()
      	     << " not found in '" << jBFile->data() << "'\n";
        return blocks.at(0);
      }
      else {
        if(blocks.at(index)->slotsCount()==0)
    	    blocks.at(index)->setSlotsBlock(blocks.at(0)->getSlotsBlock());
    	if(blocks.at(index)->songsCount()==0)
    	    blocks.at(index)->setSongsBlock(blocks.at(0)->getSongsBlock());
        blocks.at(index)->init();
    	return blocks.at(index);
      }
    }
  }
  else {
    cerr << " Position " << index << " OUT OF BOUNDS!!! \n";
    return blocks.at(0);
  }
}


JabatoStrip *
JabatoSchedule::getCurrentStrip() {
  return &currentStrip;
}


uint
JabatoSchedule::blocksCount() {
  return blocks.count();
}

const int
JabatoSchedule::getBlockId(uint i) {
  if(i<blocks.count())
    return blocks.at(i)->getId();
  else
    return 0;
}


const char
*JabatoSchedule::getBlockName(uint i) {
  if(i<blocks.count())
    return blocks.at(i)->getName();
  else
    return NULL;
}

int
JabatoSchedule::getBlockPosition(int dow, int hh, int mm) {
  DEBUG(fprintf(stderr, "JabatoSchedule::getBlockPosition(%d, %d, %d)\n",dow,hh,mm));

  // IF MOMENT REQUESTED IS BETWEEN CURRENT STRIP AVOID SEARCHING
  if(!currentStrip.withinStrip(dow,hh,mm)) {
          scheduleH->find(dow, hh, mm, &currentStrip);

	  try
	  {
		  parser->setContentHandler(scheduleH);
		  parser->setErrorHandler(scheduleH);
			parser->parse(jSFile->data());
	  }
  	catch (const XMLException& e)
	  {
	    cerr << "Error during parsing: '" << jSFile->data() << "'\n" << "Exception message is:  \n"
  	       << StrX(e.getMessage()) << "\n" ;
	    return -1;
  	}
  	catch (...)
  	{
    	cerr << "\nUnexpected exception during parsing: '" << jSFile->data() << "'\n";
    	return -1;
	  }
  	if(scheduleH->error()) {
    	cerr << "\nError during parsing: '" << jSFile->data() << "'\n";
    	return -1;
  	}
  }
 	
  uint i=0;
    	
  while ((i<blocks.count()) && 
	 (blocks.at(i)->getId()!=currentStrip.blockId)) i++;

  if((i<blocks.count()) && (blocks.at(i)->getId()==currentStrip.blockId))
   	return i;
  else
   	return -1;
}


void
JabatoSchedule::checkTasks(int d, int h, int m) {
  scheduleH->checkScheduledTasks(d,h,m);
}
