#include "jabatoSchedule.h"

#define HOURS_SHIFT 6

JabatoTask::JabatoTask() {
  shellCmd = 0;
  waiting = true;
}

JabatoTask::~JabatoTask() {
  if(shellCmd) delete shellCmd;
}


void
JabatoTask::checkTask(int d, int h, int m) {
  if(((d==dow)||(dow==0))&&(h==hour)&&(m==minute)) {
    if(waiting) {
      cout << "EXECLP: " << shellCmd << "\n";
      waiting = false;
      if(!fork()) {
        execlp(shellCmd,shellCmd,NULL);
	exit(0);
      }
    }
  }
  else
    waiting = true;
}



jabatoScheduleHandler::jabatoScheduleHandler()
{

  //I BET YOU KNOW HOW TO DO THIS MUCH BETTER. THEY ARE USED AS SIMPLE CONSTANTS!!
  SCHEDULE = XMLString::transcode("JABATOSCHEDULEFILE");
  STATIONNAME = XMLString::transcode("STATIONNAME");
  AUTORECORDING = XMLString::transcode("AUTORECORDING");
  SAMPLINGFREQUENCY = XMLString::transcode("SAMPLINGFREQUENCY");

  DAY = XMLString::transcode("DAY");
  DAYOFTHEWEEK = XMLString::transcode("DAYOFWEEK");

  TIMESTRIP = XMLString::transcode("TIMESTRIP");
  BEGINATHOUR = XMLString::transcode("BH");
  BEGINATMINUTE = XMLString::transcode("BM");
  ENDATHOUR = XMLString::transcode("EH");
  ENDATMINUTE = XMLString::transcode("EM");
  BLOCKID = XMLString::transcode("BLOCKID");
  RECORDINGSTRIP = XMLString::transcode("RECORDINGSTRIP");

  EXECAT = XMLString::transcode("EXECAT");
  HOUR = XMLString::transcode("HOUR");
  MINUTE = XMLString::transcode("MINUTE");
  SHELLCMD = XMLString::transcode("SHELLCMD");

  tasks.setAutoDelete(TRUE);
  jabatoStrip = NULL;
  m_found = false;
}

jabatoScheduleHandler::~jabatoScheduleHandler()
{
  delete SCHEDULE;
  delete STATIONNAME;

  delete DAY;
  delete DAYOFTHEWEEK;

  delete TIMESTRIP;
  delete BEGINATHOUR;
  delete BEGINATMINUTE;
  delete ENDATHOUR;
  delete ENDATMINUTE;
  delete BLOCKID;
  delete RECORDINGSTRIP;

  delete EXECAT;
  delete HOUR;
  delete MINUTE;
  delete SHELLCMD;

  //	stationName SHOULD BE DELETED BY THE CALLER;
}


void
jabatoScheduleHandler::findStationName(){
  lookingFor = JABATONAME;
  jabatoStrip = 0;
  readingDay = false;
  m_found = false;
  m_error = false;
}

char *
jabatoScheduleHandler::getStationName() {
  return stationName;
}

void
jabatoScheduleHandler::find(int dow, int hh, int mm, JabatoStrip *strip){
  day = dow;
  moment = (hh<<HOURS_SHIFT) + mm;
  jabatoStrip = strip;
  lookingFor = JABATOSTRIP;
  readingDay = false;
  m_found = false;
  m_error = false;
}

bool
jabatoScheduleHandler::found(){
  return m_found;
}

bool
jabatoScheduleHandler::autoRecMode(){
  return autoRecording;
}

uint
jabatoScheduleHandler::samplingFreq(){
  return samplingFrequency;
}

void
jabatoScheduleHandler::loadScheduledTasks(){
  tasks.clear();
  lookingFor = SCHEDULEDTASKS;
  readingDay = false;
  m_found = false;
  m_error = false;
}

void
jabatoScheduleHandler::checkScheduledTasks(int d, int h, int m) {
  for(uint i=0;i<tasks.count();i++) {
    tasks.at(i)->checkTask(d,h,m);
  }
}



bool
jabatoScheduleHandler::error(){
  return m_error;
}


// ---------------------------------------------------------------------------
//  jabatoScheduleHandler: Overrides of the SAX ErrorHandler interface
// ---------------------------------------------------------------------------
void jabatoScheduleHandler::error(const SAXParseException& e)
{
    cerr << "\nError at file " << StrX(e.getSystemId())
		     << ", line " << e.getLineNumber()
    		 << ", char " << e.getColumnNumber()
         << "\n  Message: " << StrX(e.getMessage()) << endl;
    m_found = false;
    m_error = true;
}

void jabatoScheduleHandler::fatalError(const SAXParseException& e)
{
    cerr << "\nFatal Error at file " << StrX(e.getSystemId())
    		 << ", line " << e.getLineNumber()
	    	 << ", char " << e.getColumnNumber()
         << "\n  Message: " << StrX(e.getMessage()) << endl;
  m_found = false;
  m_error = true;
}

void jabatoScheduleHandler::warning(const SAXParseException& e)
{
  cerr << "\nWarning at file " << StrX(e.getSystemId())
  		 << ", line " << e.getLineNumber()
	  	 << ", char " << e.getColumnNumber()
       << "\n  Message: " << StrX(e.getMessage()) << endl;
  m_found = false;
  m_error = true;
}

// ---------------------------------------------------------------------------
//  jabatoScheduleHandler: Overrides of the SAX DocumentHandler interface
// ---------------------------------------------------------------------------

void jabatoScheduleHandler::startDocument() {
  // Possible default block begining
  endHour = 0;
  endMinute = 0;
  end = 0;
}


void jabatoScheduleHandler::endElement(const XMLCh* const uri,
								 const XMLCh* const localname,
								 const XMLCh* const qname)
{
  if (readingDay)
    if(XMLString::compareIString(qname, DAY)==0) {
      readingDay = false;
      if(!m_found) {
        jabatoStrip->blockId = 0;               // DEFAULT BLOCK
        jabatoStrip->dayOfTheWeek = day;
        jabatoStrip->beginAtHour = endHour;     //  Begins at last strip end
        jabatoStrip->beginAtMinute = endMinute; //  and
        jabatoStrip->endAtHour = 24;            //  ends with the day
        jabatoStrip->endAtMinute = 0;
        begin = end;
        end = (24 << HOURS_SHIFT);
        m_found = true;
      }
    }
}


void jabatoScheduleHandler::startElement(const   XMLCh* const    uri,
									const   XMLCh* const    localname,
									const   XMLCh* const    qname,
                  const   Attributes&		attr)
{
  if(lookingFor==JABATOSTRIP) {
    uint ui;
    if(!m_found) {
      // LOOK FOR BLOCK IN REGULAR DAYS (IDs 1-7) AND ALL WEEK BLOCKS (DAY=0)
      try {

        if((XMLString::compareIString(qname, TIMESTRIP)==0) && readingDay) {

          XMLString::textToBin(attr.getValue(BEGINATHOUR), beginHour);
          begin = (beginHour << HOURS_SHIFT);
          XMLString::textToBin(attr.getValue(BEGINATMINUTE), beginMinute);
          begin += beginMinute;

          if(begin > moment) {   		
            uint aux;
            aux = begin;
            begin = end;
            end = aux;
            jabatoStrip->blockId = 0;               // DEFAULT BLOCK
            jabatoStrip->dayOfTheWeek = day;
            jabatoStrip->beginAtHour = endHour;     //  Begins at last strip end
            jabatoStrip->beginAtMinute = endMinute; //  and
            jabatoStrip->endAtHour = beginHour;     //  ends at current begining
            jabatoStrip->endAtMinute = beginMinute;
            m_found = true;
          }
          else {
            XMLString::textToBin(attr.getValue(ENDATHOUR), endHour);
            end = (endHour << HOURS_SHIFT);
            XMLString::textToBin(attr.getValue(ENDATMINUTE), endMinute);
            end += endMinute;

            if ((begin <= moment) && (moment < end)) {
              XMLString::textToBin(attr.getValue(BLOCKID), ui);
              jabatoStrip->blockId = ui;
              try {
                XMLString::textToBin(attr.getValue(RECORDINGSTRIP), ui);
                if(ui==1)
                  jabatoStrip->recordingStrip = true;
                else
                  jabatoStrip->recordingStrip = false;
              }
              catch (...) {
                jabatoStrip->recordingStrip = false;
              }
              jabatoStrip->dayOfTheWeek = day;
              jabatoStrip->beginAtHour = beginHour;
              jabatoStrip->beginAtMinute = beginMinute;
              jabatoStrip->endAtHour = endHour;
              jabatoStrip->endAtMinute = endMinute;
              m_found = true;
            }
          }
        }
        else if(XMLString::compareIString(qname, DAY)==0) {
          if (XMLString::textToBin(attr.getValue(DAYOFTHEWEEK), ui))
            readingDay = ((int)ui==day);
        }
      }
      catch (...){
        if(stationName) delete stationName;
        m_found = readingDay = false;
        cerr << "ERROR IN jSF FILE IN <" << StrX(qname) << "> TAG\n";
      }
    }
    else if (jabatoStrip->blockId == 0) {
      // IF FOUND DEFAULT BLOCK LOOK FOR OTHER IN ALL WEEK BLOCKS (ID:0)
      if(XMLString::compareIString(qname, DAY)==0)
      {
        if (XMLString::textToBin(attr.getValue(DAYOFTHEWEEK), ui))
          readingDay = (ui==0);
      }
      else if((XMLString::compareIString(qname, TIMESTRIP)==0) &&
              (readingDay)) {
        uint thisBegin;
        uint thisEnd;

        XMLString::textToBin(attr.getValue(BEGINATHOUR), beginHour);
        thisBegin = (beginHour << HOURS_SHIFT);
        XMLString::textToBin(attr.getValue(BEGINATMINUTE), beginMinute);
        thisBegin += beginMinute;
        XMLString::textToBin(attr.getValue(ENDATHOUR), endHour);
        thisEnd = (endHour << HOURS_SHIFT);
        XMLString::textToBin(attr.getValue(ENDATMINUTE), endMinute);
        thisEnd += endMinute;


        if ((thisBegin <= moment) && (moment < thisEnd)) {
          uint ui;
          XMLString::textToBin(attr.getValue(BLOCKID), ui);
          jabatoStrip->blockId = ui;
          try {
            XMLString::textToBin(attr.getValue(RECORDINGSTRIP), ui);
            if(ui==1)
              jabatoStrip->recordingStrip = true;
            else
              jabatoStrip->recordingStrip = false;
          }
          catch (...) {
            jabatoStrip->recordingStrip = false;
          }

          jabatoStrip->dayOfTheWeek = day;
          jabatoStrip->beginAtHour = beginHour;
          jabatoStrip->beginAtMinute = beginMinute;
          jabatoStrip->endAtHour = endHour;
          jabatoStrip->endAtMinute = endMinute;
          readingDay = false;
        }
        // MAYBE WE CAN MAKE THE DEFAULT BLOCK SHORTER
        //  -- DELAYING THE END
        else if ((moment >= thisEnd) && (begin < thisEnd)) {
          begin = thisEnd;
          jabatoStrip->beginAtHour = endHour;
          jabatoStrip->beginAtMinute = endMinute;
        }
        //  -- FORWARDING THE END
        else if ((thisBegin > moment) && (end > thisBegin)) {
          end = thisBegin;
          jabatoStrip->endAtHour = beginHour;
          jabatoStrip->endAtMinute = beginMinute;
        }
      }
    }
  }
  else if(lookingFor==SCHEDULEDTASKS) {
    if(XMLString::compareIString(qname, DAY)==0) {
      uint ui;
      if (XMLString::textToBin(attr.getValue(DAYOFTHEWEEK), ui))
        day = (int)ui;
    }
    else if(XMLString::compareIString(qname, EXECAT)==0) {
      try {
	      uint ui;
	      JabatoTask *task = new JabatoTask();
        task->dow = day;
        XMLString::textToBin(attr.getValue(HOUR), ui);
        task->hour = (int)ui;
        XMLString::textToBin(attr.getValue(MINUTE), ui);
        task->minute = (int)ui;
        task->shellCmd = XMLString::transcode(attr.getValue(SHELLCMD));
        tasks.append(task);
        cerr << "APPEND TASK " << task->shellCmd << " DOW:"
             << task->dow << " HOUR:" << task->hour << " MINUTE:" << task->minute << "\n";
      }
      catch (...) {
        cerr << "ERROR IN jSF FILE IN <" << StrX(qname) << "> TAG\n";
      }
    }
  }
  else if ((lookingFor==JABATONAME) && !m_found) {
    if(XMLString::compareIString(qname, SCHEDULE)==0) {
      uint ui;
      try {
        XMLString::textToBin(attr.getValue(AUTORECORDING), ui);
        if(ui==1)
          autoRecording = true;
        else
          autoRecording = false;
        }
      catch (...) {
        autoRecording = false;
      }
      try {
        XMLString::textToBin(attr.getValue(SAMPLINGFREQUENCY), 
	                     samplingFrequency);
      }
      catch (...) {
        samplingFrequency = 0;
      }
      stationName = XMLString::transcode(attr.getValue(STATIONNAME));
      m_found = true;
    }
  }

}

